@@ -5,7 +5,6 @@
 
 from contextlib import AbstractContextManager
 from itertools import cycle, islice
-from types import ModuleType
 from typing import Annotated, Any, Literal, Optional
 
 from corgy import Corgy
@@ -13,15 +12,14 @@ from corgy import Corgy
 _WRAPPED_NAMES = ("mpl", "plt", "sns")
 __all__ = ("MatWrap", "PlottingArgs", "Plot") + _WRAPPED_NAMES
 
-mpl: ModuleType
-plt: ModuleType
-sns: ModuleType
+mpl: Any
+plt: Any
+sns: Any
 
 
 def __getattr__(name):
     if name in _WRAPPED_NAMES:
-        attr = getattr(MatWrap, name)
-        return attr.__func__(MatWrap)
+        return getattr(MatWrap, name)
     raise AttributeError
 
 
@@ -197,6 +195,9 @@ class MatWrap:
 class PlottingArgs(Corgy):
     """Plotting arguments that can be added to `ArgumentParser` instances.
 
+    `MatWrap.configure` is called with the chosen arguments when an instance of this
+    class is created.
+
     Usage::
 
         >>> arg_parser = ArgumentParser(add_help=False, formatter_class=Corgy)
@@ -231,6 +232,12 @@ class PlottingArgs(Corgy):
     font: Annotated[str, "font for plots"] = "Latin Modern Roman"
     backend: Annotated[str, "matplotlib backend"] = "pgf"
 
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        MatWrap.configure(
+            context=self.context, style=self.style, font=self.font, backend=self.backend
+        )
+
 
 class Plot(AbstractContextManager):
     """Wrapper around a single matplotlib plot.
